#version 120
#extension GL_3DL_array_objects : enable

uniform sampler2D u_linear_depth_tex;
uniform sampler2D u_noise_tex;

varying vec3 v_view_ray;
varying vec2 v_tex_coord;
varying vec2 v_noise_coord;

// some random directions inside a sphere, 
// distrubuted in a way that when summed up the 
// result is nearly (0, 0, 0).
vec3 sphere_directions[8] = vec3[8](
	vec3(-0.0246206, 0.0117333, -0.999628),
	vec3(0.0277852, 0.168455, -0.985318),
	vec3(-0.306463, 0.17691, 0.935298),
	vec3(-0.769876, -0.134997, 0.623752),
	vec3(0.792168, 0.399472, -0.461403),
	vec3(0.438936, -0.584636, 0.682302),
	vec3(-0.756404, 0.623771, -0.196882),
	vec3(0.626924, -0.6764, 0.386588)
);

float unpack_float_from_vec4(const vec4 value)
{
	const vec4 bitSh = vec4(1.0 / (256.0 * 256.0 * 256.0), 1.0 / (256.0 * 256.0), 1.0 / 256.0, 1.0);
	return(dot(value, bitSh));
}

float linear_depth(vec2 sample)
{
	return unpack_float_from_vec4(texture2D(u_linear_depth_tex, sample));
	//return texture2D(u_linear_depth_tex, sample).r;
}

float ssao()
{
	const float c_radius = 0.04;
	const float c_dist_scale = 1.0;

	// linear z in [0, 1]
	float lz = linear_depth(v_tex_coord);
	
	// random plane around which we reflect the sphere coordinates to get randomized sampling.
	vec3  rp = texture2D(u_noise_tex, v_noise_coord).rgb * 2.0 - 1.0;
		
	float block = 0.0;

	// accumulate blocking factor for all samples
	for (int i = 0; i < 8; ++i) {
		vec3 o = c_radius * reflect(rp, sphere_directions[i]);
		float rz = linear_depth(v_tex_coord + o.xy);
		float zd = (rz - lz) * v_view_ray.z * c_dist_scale;
		block += step(0.0, zd) / (1.0 + zd * zd);
	}

	return 1.0 - block / 8.0;
}

void main()
{
	gl_FragColor = vec4(vec3(ssao()), 1.0);
}
